#!/usr/bin/env python3

# initialise dictionary to store bar heights
height = {}
height["prey"] = None
height["wcereal"] = None
height["fcereal"] = None

# set divisions of bar height
step = 10

# loop through all permutations
# (bar height 0/step to step/step)
for prey in range(step + 1):
    for wcereal in range(step + 1):
        for fcereal in range(step + 1):

            # generate bar heights for this iteration
            height["prey"] = float(prey / step * 200)
            height["wcereal"] = float(wcereal / step * 200)
            height["fcereal"] = float(fcereal / step * 200)

            # generate output filename for this iteration
            outname = "histogram-" + str(prey).zfill(2) + \
                    str(wcereal).zfill(2) + str(fcereal).zfill(2) + \
                    ".svg"

            print("Generating " + outname)

            # initialise context variable
            nextfix = None

            # open template and start this iteration
            with open('generate-template.svg', 'r') as inputfile, \
                    open(outname, 'w') as outputfile:

                for line in inputfile:

                    # initialise the control variables
                    splitline = line.split('"')

                    # if we hit the generation line, replace it
                    if (line == '<!-- Created with Inkscape (http://www.inkscape.org/) -->\n'):
                        line = '<!-- Created with HG2A Python script -->\n'

                    # if we hit an id line, update the context
                    elif (line.strip().startswith('id="')):
                        nextfix = splitline[1]

                    # if we hit a height line, make a fix (if
                    # appropriate)
                    elif (line.strip().startswith('height="')):

                        # if we are in the context of a bar element,
                        # set the height
                        if (nextfix in ("prey", "wcereal", "fcereal")):
                            bar_height = '"' + \
                                    "{0:.2f}".format(height[nextfix]) + \
                                    '"'
                            line = splitline[0] + bar_height + \
                                    splitline[2]

                    # if we hit a y-position line, make a fix (if
                    # appropriate)
                    elif (line.strip().startswith('y="')):

                        # if we are in the context of a bar element,
                        # set the y-postion
                        if (nextfix in ("prey", "wcereal", "fcereal")):
                            bar_y = '"' + "{0:.2f}".format(200 - \
                                    height[nextfix]) + '"'
                            line = splitline[0] + bar_y + \
                                    splitline[2]

                    # for all other cases, the line will stay the same
                    else:
                        pass

                    # write the reviewed line to the output file
                    # for the current iteration
                    outputfile.write(line)
