package fhg2a.relogo

import static repast.simphony.relogo.Utility.*;
import static repast.simphony.relogo.UtilityG.*;
import repast.simphony.relogo.Plural;
import repast.simphony.relogo.Stop;
import repast.simphony.relogo.Utility;
import repast.simphony.relogo.UtilityG;
import repast.simphony.relogo.schedule.Go;
import repast.simphony.relogo.schedule.Setup;
import fhg2a.ReLogoTurtle;

/**
 * A turtle used to display food and population levels. When updated these
 * turtles inspect the food and population levels of the patch they are on
 * and update their shape to reflect food levels using a three-bar histogram.
 * They update their label to reflect population levels as a count of
 * tribes.
 * 
 */
class DisplayHistogram extends ReLogoTurtle {

	/**
	 * Update shape and label based on food and population levels of underlying patch.
	 *
	 */
	def update() {

		//calculate food levels on a scale from 0 to 10
		def levelPrey = String.format("%02d", (int) Math.round(patchHere().prey.current / 2400.0 * 10.0))
		def levelWildCereal = String.format("%02d", (int) Math.round(patchHere().wildCereal.current / 180000.0 * 10.0))
		def levelFarmedCereal = String.format("%02d", (int) Math.ceil(patchHere().farmedCereal.current / 21000.0 * 10.0))

		//generate histogram string based on food levels
		def histogramName = "histogram-" + levelPrey + levelWildCereal + levelFarmedCereal

		//update shape using histogram string
		setShape("histogram-" + levelPrey + levelWildCereal + levelFarmedCereal)

		//label histogram with number of tribes
		//(blank if no tribes present, X is 10 or more tribes are present)
		//FIXME: this should be individuals not tribes (update class comment too)
		def total = count(tribesHere())
		if (total == 0)
			setLabel(" ")
		else if (total < 10)
			setLabel((int) total)
		else
			setLabel("X")

	}
	
}
