package fhg2a.relogo

import groovy.lang.MetaClass;

/**
 * Tribe that relies on both wild cereal and prey for
 * sustenance, and has perfect global knowledge. Divides
 * annual foraging time proportionately between the
 * two resources according to their net acquisition rates.
 * 
 */
class HunterGatherer extends Tribe {

	/**
	 * Forage one year's worth of prey and cereal. Decrease population
	 * if enough food is not gathered.
	 */
	 def forage(){

		 //get acquisition rates for all food types
		 def preyNAR = patchHere().prey.netAcquisitionRate()
		 def wildCerealNAR = patchHere().wildCereal.netAcquisitionRate()

		 //initialise measures of energy
		 def energyRequired = populationSize * 2000.0 * 365.0
		 def energyGained = 0.0

		 //calculate proportions of effort for prey based on energy efficiency
		 //then get energy from prey
		 def totalNAR = preyNAR + wildCerealNAR
		 def portion = preyNAR / totalNAR
		 energyGained += patchHere().handlePrey((portion * energyRequired), populationSize)
		 energyRequired = Math.max(0.0, energyRequired - energyGained)

		 System.out.println("Prey NAR " + preyNAR + " wild cereal NAR " + wildCerealNAR + ": total " + totalNAR + ": prey ratio" + portion)
		 System.out.println("Energy gained (prey) " + energyGained + "/" + energyRequired)

		 //recalculate proportions of effort for wild cereal based on energy efficiency
		 //and energy needs based on prey success, then get energy from wild cereal
		 energyGained += patchHere().handleWildCereal(energyRequired, populationSize)
		 energyRequired = Math.max(0.0, energyRequired - energyGained)

		 System.out.println("Energy gained (prey + cereal) " + energyGained + "/" + energyRequired)
		 		 
		 //cull tribe population according to total energy obtained from all sources
		 cull(energyGained)
	 }
	
	/**
	 * Move tribe to the best patch on the grid (including patch it is on).
	 * Patches are ranked by prey density then cereal density.
	 * 
	 */
	def move() {

		//get the set of patches that have the maximum
		//PREY density of all patches on the grid
		def bestPreySet = patches().withMax() {
			it.prey.current / 300.0
		}

		//get subset of this set that have the maximum
		//WILD CEREAL density
		def bestWildCerealSet = bestPreySet.withMax() {
			it.wildCereal.current / 300.0
		}

		//move to the first patch in the subset
		def patchNow= patchHere()
		moveTo(bestWildCerealSet.get(0))
		def patchAfter = patchHere()

		// count migration (movement between habitats) if one took place
		if (!patchNow.habitat.equalsIgnoreCase(patchAfter.habitat)) {
			migrations++
		}
	}
}
