package fhg2a.relogo

import groovy.lang.MetaClass;

/**
 * Tribe that relies on wild cereal, prey and farmed cereal
 * for sustenance, but has only limited, local knowledge.
 * Divides annual foraging time between all food types based
 * on their relative net acquisition rates.
 *
 */
class RealisticFreeDistribution extends IdealFreeDistribution {

	/**
	 * Move tribe to the best patch in its immediate neighborhood
	 * (including patch it is on). Patches are ranked by prey
	 * density then cereal density.
	 *
	 */
	 def move() {

		 //get the set of neighborhood patches that have the maximum
		 //PREY density of all patches in the neighborhood
		 def bestPreySet = neighbors().withMax() {
			 it.prey.current / 300.0
		 }
 
		 //get subset of this set that have the maximum
		 //WILD CEREAL density
		 def bestWildCerealSet = bestPreySet.withMax() {
			 it.wildCereal.current / 300.0
		 }

		//move to the first patch in the subset
		def patchNow= patchHere()
		moveTo(bestWildCerealSet.get(0))
		def patchAfter = patchHere()

		//count a migration (movement between habitats) if it occurred
		if(!patchNow.habitat.equalsIgnoreCase(patchAfter.habitat)){
			migrations++
		}
	}
}
