package fhg2a.relogo

import static repast.simphony.relogo.Utility.*;
import static repast.simphony.relogo.UtilityG.*;
import repast.simphony.relogo.Stop;
import repast.simphony.relogo.Utility;
import repast.simphony.relogo.UtilityG;
import repast.simphony.relogo.schedule.Go;
import repast.simphony.relogo.schedule.Setup;
import fhg2a.ReLogoObserver;

/**
 * Simulation controller. Sets up the interface and experiment,
 * initiates the simulation and executes every step.
 *
 */
class UserObserver extends ReLogoObserver {

	/**
	 * Initialises grid when user clicks setup button. Grid is divided
	 * into three habitats and populated appropriate amounts of food.
	 * Requested number of tribes are scattered randomly across the
	 * grid.
	 * 
	 */
	@Setup
	def setup() {

		//clear grid of all tribes
		clearAll()

		//grid has 3 equal width belts of food resource lushness
		//calculate belt width from grid width
		def totalGridWidth = getMaxPxcor() - getMinPxcor() + 1
		def beltWidth = Math.floor(totalGridWidth / 3.0).intValue()

		//FIXME: replace array below with enums/something groovy

		//3x3 array defining food richness for three food sources in three belts
		//(units per square kilometer)
		def capacity = new Double[3][3]

		//lush belt
		capacity[0][0] = 8.0
		capacity[0][1] = 600.0
		capacity[0][2] = 600.0

		//middle belt
		capacity[1][0] = 5.4
		capacity[1][1] = 400.0
		capacity[1][2] = 400.0

		//desert belt
		capacity[2][0] = 2.6
		capacity[2][1] = 200.0
		capacity[2][2] = 200.0

		//initialise food source densities for all patches on grid
		//and assign habitat to patch as a string
		for (belt in 0..2) {
			for (beltColumn in 0..(beltWidth-1)) {
				for (y in getMinPycor()..getMaxPycor()) {
					def x = getMinPxcor() + beltColumn + (beltWidth * belt)

					def preyCapacity = capacity[belt][0]
					def wildCerealCapacity = capacity[belt][1]
					def farmedCerealCapacity = capacity[belt][2]
					patch(x, y).setCarryingCapacity(preyCapacity, wildCerealCapacity, farmedCerealCapacity)
					patch(x, y).setPcolor(green())

					if (belt == 0) {
						patch(x,y).habitat = "Lush"
					}
					else if (belt == 1) {
						patch(x,y).habitat = "Middle"
					}
					else if (belt == 2) {
						patch(x,y).habitat = "Desert"
					}
				}
			}
		}

		//if histogram drawing has been selected, create static display turtles
		//these will be updated later to reflect food and population levels
		if (drawOption == "Histogram") {
			def xHistogram = 0
			def yHistogram = 0
			createDisplayHistograms(21*21){
				setxy(xHistogram, yHistogram)
				xHistogram += 1
				if (xHistogram > 20) {
					xHistogram = 0
					yHistogram += 1
				}
			}
		}

		//draw patches according to user-selected drawing method
		redrawPatches()

		//create number of specified tribes (by slider and drop-down on user panel)
		//and assign each one to a random location on grid
		if (experimentChoice=="Gatherer") {
			createGatherers(numberOfTribes) {
				setxy(randomXcor(), randomYcor())
			}
		}

		else if (experimentChoice=="Hunter-gatherer") {
			createHunterGatherers(numberOfTribes) {
				setxy(randomXcor(), randomYcor())
			}
		}

		else if (experimentChoice=="Ideal Free Distribution") {
			createIdealFreeDistributions(numberOfTribes){
				setxy(randomXcor(), randomYcor())
			}
		}

		else if (experimentChoice=="Realistic Free Distribtion") {
			createRealisticFreeDistributions(numberOfTribes) {
				setxy(randomXcor(), randomYcor())
			}
		}
	}

	/**
	 * Each epoch, execute activity for all tribes and all food resources
	 * on the grid.
	 * 
	 */
	@Go
	def go(){

		//deactivate farming at start of epoch
		ask(patches()){
			deactivateFarmedCereal()
		}

		//one year cycle of activity for each tribe
		ask(tribes()){
			step(it.getTurtleType())
		}

		//replenish growable food for each patch
		ask(patches()){
			replenishWildCereal()
			replenishPrey()

			//populate from viable neighbours (prey and wild cereal only)
			crosspopulateFood()
		}

		//redraw patches to display new food and population levels
		redrawPatches()
	}

	/**
	 * Redraw patches based on user-selected drawing method.
	 * 
	 */
	def redrawPatches() {

		if (drawOption == "Histogram"){
			drawHistogram()
		}

		else if (drawOption == "Patch colour") {
			drawPatchColour()
		}
	}

	/**
	 * Display amount of wild cereal available, by colouring each patch
	 * on a false colour spectrum.
	 * 
	 */
	def drawPatchColour() {

		//iterate over patches and update colour
		ask(patches()){
			def proportion = wildCereal.current/180000 // FIXME: review this value
			setPcolor(proportion * 140)
		}
	}

	/**
	 * Display population and amount of food available, by displaying population
	 * label and three-bar resource histogram.
	 * 
	 */
	def drawHistogram() {

		//iterate over static display turtles and update
		ask(displayHistograms()) {
			update()
		}
	}
}
